#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const fs_1 = __importDefault(require("fs"));
const prettier_1 = __importDefault(require("prettier"));
const commander_1 = require("commander");
commander_1.program.option("-p, --path <string>");
commander_1.program.parse();
const options = commander_1.program.opts();
const generateType = ({ name, schema }) => {
    const convertTypes = (type, currProp) => {
        if (type.includes("#")) {
            return `${type.split("/schemas/")[1]}`;
        }
        if (type === "integer")
            return "number";
        if (type === "array") {
            if ("$ref" in currProp.items) {
                return `${currProp.items["$ref"].split("/schemas/")[1]}[]`;
            }
            else if ("type" in currProp.items) {
                return `${currProp.items.type}[]`;
            }
        }
        return type;
    };
    const reduceProperties = (properties) => {
        return Object.keys(properties).reduce((prev, curr) => {
            return (prev +
                `
      ${curr}: ${convertTypes(properties[curr].type || properties[curr]["$ref"], properties[curr])};`);
        }, "");
    };
    return `
  interface ${name} {
    ${reduceProperties(schema.properties)}
  }
  `;
};
(() => __awaiter(void 0, void 0, void 0, function* () {
    const swaggerData = yield axios_1.default.get(options.path);
    const schemas = swaggerData.data.components.schemas;
    const typesFileContent = Object.keys(schemas).reduce((prev, curr) => {
        return prev + generateType({ name: curr, schema: schemas[curr] });
    }, "");
    const finalData = prettier_1.default.format(`// This file is auto-generated (do not modify) ${typesFileContent}`, {
        parser: "babel",
    });
    console.log({ options });
    fs_1.default.writeFileSync("test.ts", finalData);
}))();
